package com.googlecode.alfresco.repository.node;

import java.io.Serializable;
import java.util.Collection;

import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.repository.NodeService;
import org.alfresco.service.namespace.QName;

/**
 * Defines helper operations for working with repository nodes. Intended as a companion to the Alfresco
 * {@link NodeService}.
 *
 * @author Laurens Fridael
 *
 */
public interface NodeHelper {
	/**
	 * Obtains the path of a node in XPath format.
	 *
	 * @param nodeRef
	 *            The NodeRef.
	 * @return The NodeRef path.
	 * @see PathHelper
	 */
	public String getPath(NodeRef nodeRef);

	/**
	 * Obtains the values for a multi-valued property of a node. If the property is single-valued (i.e. multiplicity of
	 * one), this method wraps the value in a Collection with a single element.
	 *
	 * @param nodeRef
	 *            The NodeRef.
	 * @param property
	 *            The property.
	 * @return The property values as a Collection or null if the property's value is null or the node does not contain
	 *         the property.
	 */
	public <T extends Serializable> Collection<T> getPropertyValues(NodeRef nodeRef, QName property);

	/**
	 * Determines if a node is of a given type.
	 *
	 * @param nodeRef
	 *            The NodeRef.
	 * @param type
	 *            The type QName.
	 * @return True if the node is of the given type, false if not.
	 */
	public boolean isOfType(NodeRef nodeRef, QName type);
}
