package com.googlecode.alfresco.repository.node.impl;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Collection;

import org.alfresco.service.cmr.dictionary.DictionaryService;
import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.repository.NodeService;
import org.alfresco.service.cmr.repository.Path;
import org.alfresco.service.namespace.QName;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.Assert;

import com.googlecode.alfresco.repository.node.NodeHelper;
import com.googlecode.alfresco.repository.node.PathHelper;

public class NodeHelperImpl implements NodeHelper {

	private NodeService nodeService;

	private DictionaryService dictionaryService;

	private PathHelper pathHelper;

	@Required
	public void setNodeService(final NodeService nodeService) {
		Assert.notNull(nodeService, "NodeService cannot be null.");
		this.nodeService = nodeService;
	}

	protected NodeService getNodeService() {
		return nodeService;
	}

	@Required
	public void setDictionaryService(final DictionaryService dictionaryService) {
		Assert.notNull(dictionaryService, "DictionaryService cannot be null.");
		this.dictionaryService = dictionaryService;
	}

	protected DictionaryService getDictionaryService() {
		return dictionaryService;
	}

	@Required
	public void setPathHelper(final PathHelper pathHelper) {
		Assert.notNull(pathHelper, "PathHelper cannot be null.");
		this.pathHelper = pathHelper;
	}

	protected PathHelper getPathHelper() {
		return pathHelper;
	}

	@Override
	public String getPath(final NodeRef nodeRef) {
		Assert.notNull(nodeRef, "NodeRef cannot be null.");
		final Path path = getNodeService().getPath(nodeRef);
		return getPathHelper().convertPathToString(path);
	}

	@SuppressWarnings("unchecked")
	@Override
	public <T extends Serializable> Collection<T> getPropertyValues(final NodeRef nodeRef, final QName property) {
		Assert.notNull(nodeRef, "NodeRef cannot be null.");
		Assert.notNull(property, "Property cannot be null.");
		final Object value = getNodeService().getProperty(nodeRef, property);
		if (value instanceof Collection) {
			return (Collection<T>) value;
		} else if (value != null) {
			return Arrays.asList((T) value);
		} else {
			return null;
		}
	}

	@Override
	public boolean isOfType(final NodeRef nodeRef, final QName type) {
		Assert.notNull(nodeRef, "NodeRef cannot be null.");
		Assert.notNull(type, "Type cannot be null.");
		return getDictionaryService().isSubClass(getNodeService().getType(nodeRef), type);
	}

}
