package com.googlecode.alfresco.repository.node.impl;

import java.util.Collection;

import org.alfresco.service.cmr.repository.Path;
import org.alfresco.service.namespace.NamespacePrefixResolver;
import org.alfresco.service.namespace.QName;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.Assert;

import com.googlecode.alfresco.repository.node.PathHelper;

public class PathHelperImpl implements PathHelper {

	private NamespacePrefixResolver namespacePrefixResolver;

	private PathEncoder pathEncoder = new ISO9075PathEncoder();

	@Required
	public void setNamespacePrefixResolver(final NamespacePrefixResolver namespacePrefixResolver) {
		Assert.notNull(namespacePrefixResolver, "NamespacePrefixResolver cannot be null.");
		this.namespacePrefixResolver = namespacePrefixResolver;
	}

	protected NamespacePrefixResolver getNamespacePrefixResolver() {
		return namespacePrefixResolver;
	}

	public void setPathEncoder(final PathEncoder pathEncoder) {
		Assert.notNull(pathEncoder, "PathEncoder cannot be null.");
		this.pathEncoder = pathEncoder;
	}

	protected PathEncoder getPathEncoder() {
		return pathEncoder;
	}

	@Override
	public String convertPathToString(final Path path) {
		Assert.notNull(path, "Path cannot be null.");
		final StringBuilder str = new StringBuilder();
		for (final Path.Element element : path) {
			final Path.ChildAssocElement childAssociationElement = (Path.ChildAssocElement) element;
			if (childAssociationElement.getRef().getParentRef() != null) {
				final QName childQName = childAssociationElement.getRef().getQName();
				final String prefix = getPrefixFor(childQName);
				final String localName = getPathEncoder().encodePath(childQName.getLocalName());
				str.append(String.format("/%s:%s", prefix, localName));
			}
		}
		return str.toString();
	}

	private String getPrefixFor(final QName qname) {
		String prefix = null;
		final Collection<String> prefixes = getNamespacePrefixResolver().getPrefixes(qname.getNamespaceURI());
		if (!prefixes.isEmpty()) {
			prefix = prefixes.iterator().next();
		}
		return prefix;
	}
}
