package com.googlecode.alfresco.repository.query;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.alfresco.service.cmr.repository.StoreRef;
import org.springframework.util.Assert;


/**
 * A value object representing a Query. A query is a sequence of conditions and logical operations (AND, OR).
 *
 * @author Laurens Fridael
 * @see QueryBuilder
 */
public class Query implements Iterable<Object>, Invertable {

	/* package */final List<Object> terms = new ArrayList<Object>();

	private final StoreRef storeRef;

	private boolean inverted;

	/**
	 * Creates an instance for the default store.
	 */
	public Query() {
		this(StoreRef.STORE_REF_WORKSPACE_SPACESSTORE);
	}

	/**
	 * Creates an instance for the given store.
	 *
	 * @param storeRef
	 *            The StoreRef.
	 */
	public Query(final StoreRef storeRef) {
		Assert.notNull(storeRef, "StoreRef cannot be null.");
		this.storeRef = storeRef;
	}

	/**
	 * Creates an instance by copying another instance's terms, but using a possibly different StoreRef.
	 *
	 * @param query
	 * @param
	 */
	public Query(final Query query, final StoreRef storeRef) {
		Assert.notNull(query, "Query cannot be null.");
		Assert.notNull(storeRef, "StoreRef cannot be null.");
		this.terms.addAll(query.terms);
		this.storeRef = storeRef;
	}

	/**
	 * Copy constructor for creating an instance from another instance.
	 *
	 * @param query
	 *            The instance to create from.
	 */
	public Query(final Query query) {
		this(query, query.storeRef);
	}

	public StoreRef getStoreRef() {
		return storeRef;
	}

	public void addTerm(final Object term) {
		Assert.notNull(term, "Term cannot be null.");
		terms.add(term);
	}

	public boolean isEmpty() {
		return terms.isEmpty();
	}

	@Override
	public Iterator<Object> iterator() {
		return terms.iterator();
	}

	@Override
	public void setInverted(final boolean inverted) {
		this.inverted = inverted;
	}

	@Override
	public boolean isInverted() {
		return inverted;
	}

}
