package com.googlecode.alfresco.repository.query;

import java.util.List;

import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.search.ResultSet;
import org.alfresco.service.cmr.search.SearchParameters;
import org.alfresco.service.cmr.search.SearchService;

/**
 * Defines operations for performing searches using {@link Query} objects.
 *
 * @author Laurens Fridael
 *
 */
public interface QueryHelper {

	/**
	 * Performs a query and returns its {@link ResultSet}. Clients are responsible for managing (and more importantly:
	 * closing) the ResultSet.
	 *
	 * @param query
	 * @return The query's ResultSet.
	 */
	public ResultSet query(Query query);

	/**
	 * Performs a query and returns its {@link ResultSet}. Callers are responsible for managing (and more importantly:
	 * closing) the ResultSet.
	 *
	 * @param query
	 * @param variableResolver
	 * @return The query's ResultSet.
	 */
	public ResultSet query(Query query, VariableResolver variableResolver);

	/**
	 * Performs a query and returns the results as a list of {@link NodeRef}s.
	 * <p>
	 * Implementations are responsible for closing the underlying ResultSet.
	 *
	 * @param query
	 * @return The results as a List of NodeRefs.
	 */
	public List<NodeRef> queryNodeRefs(Query query);

	/**
	 * Performs a query and returns the results as a list of {@link NodeRef}s.
	 * <p>
	 * Implementations are responsible for closing the underlying ResultSet.
	 *
	 * @param query
	 * @param variableResolver
	 * @return The results as a List of NodeRefs.
	 */
	public List<NodeRef> queryNodeRefs(Query query, VariableResolver variableResolver);

	/**
	 * Converts a given {@link Query} to {@link SearchParameters} for use with the Alfresco {@link SearchService} API.
	 *
	 * @param query
	 * @return The SearchParameters
	 */
	public SearchParameters createSearchParameters(Query query);

	/**
	 * Converts a given {@link Query} to {@link SearchParameters} for use with the Alfresco {@link SearchService} API.
	 *
	 * @param query
	 * @param variableResolver
	 * @return The SearchParameters
	 */
	public SearchParameters createSearchParameters(Query query, VariableResolver variableResolver);
}
