package com.googlecode.alfresco.repository.query;

/**
 * Defines a Strategy for resolving {@link Variable}s.
 * <p>
 * If a variable cannot be resolved, callers should omit the variable's condition and any preceding logical operation
 * from the generated query.
 *
 * @author Laurens Fridael
 */
public interface VariableResolver {

	/**
	 * Resolves a given Variable.
	 *
	 * @param name
	 *            The variable name
	 * @return The resolved value or null if the value could not be resolved.
	 */
	public Object resolveVariable(String name);
}
