package com.googlecode.alfresco.repository.query.impl;

import java.util.List;

import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.service.cmr.search.ResultSet;
import org.alfresco.service.cmr.search.SearchParameters;
import org.alfresco.service.cmr.search.SearchService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.Assert;

import com.googlecode.alfresco.repository.query.Query;
import com.googlecode.alfresco.repository.query.QueryHelper;
import com.googlecode.alfresco.repository.query.QuerySerializer;
import com.googlecode.alfresco.repository.query.VariableResolver;

/**
 * Default {@link QueryHelper} implementation.
 *
 * @author Laurens Fridael
 *
 */
public class QueryHelperImpl implements QueryHelper {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	private SearchService searchService;

	private QuerySerializer querySerializer;

	@Required
	public void setSearchService(final SearchService searchService) {
		Assert.notNull(searchService, "SearchService cannot be null.");
		this.searchService = searchService;
	}

	protected SearchService getSearchService() {
		return searchService;
	}

	@Required
	public void setQuerySerializer(final QuerySerializer querySerializer) {
		Assert.notNull(querySerializer, "QuerySerializer cannot be null.");
		this.querySerializer = querySerializer;
	}

	protected QuerySerializer getQuerySerializer() {
		return querySerializer;
	}

	@Override
	public ResultSet query(final Query query) {
		Assert.notNull(query, "Query cannot be null.");
		return query(query, null);
	}

	@Override
	public ResultSet query(final Query query, final VariableResolver variableResolver) {
		Assert.notNull(query, "Query cannot be null.");
		final SearchParameters searchParameters = createSearchParameters(query, variableResolver);
		if (logger.isDebugEnabled()) {
			logger.debug("Performing {} query: {}",
					new Object[] { searchParameters.getLanguage(), searchParameters.getQuery() });
		}
		return getSearchService().query(searchParameters);
	}

	@Override
	public List<NodeRef> queryNodeRefs(final Query query) {
		Assert.notNull(query, "Query cannot be null.");
		return queryNodeRefs(query, null);
	}

	@Override
	public List<NodeRef> queryNodeRefs(final Query query, final VariableResolver variableResolver) {
		Assert.notNull(query, "Query cannot be null.");
		final SearchParameters searchParameters = createSearchParameters(query, variableResolver);
		if (logger.isDebugEnabled()) {
			logger.debug("Performing {} query: {}",
					new Object[] { searchParameters.getLanguage(), searchParameters.getQuery() });
		}
		final ResultSet resultSet = getSearchService().query(searchParameters);
		try {
			return resultSet.getNodeRefs();
		} finally {
			resultSet.close();
		}
	}

	@Override
	public SearchParameters createSearchParameters(final Query query) {
		return createSearchParameters(query, null);
	}

	@Override
	public SearchParameters createSearchParameters(final Query query, final VariableResolver variableResolver) {
		Assert.notNull(query);
		final SearchParameters searchParameters = new SearchParameters();
		searchParameters.addStore(query.getStoreRef());
		searchParameters.setLanguage(getQuerySerializer().getLanguage());
		searchParameters.setQuery(getQuerySerializer().serializeAsString(query, variableResolver));
		return searchParameters;
	}

}
